import subprocess
import io
import os
import yaml
import sys
import os.path

from datetime import datetime


def exec_with_envs(commands, tsuru_envs, with_shell=False, working_dir="/home/application/current"):
    envs = {env['name']: env['value'] for env in tsuru_envs}
    envs.update(os.environ)
    if not os.path.exists(working_dir):
        working_dir = "/"
    for command in commands:
        status = subprocess.Popen(command, shell=with_shell, cwd=working_dir, env=envs).wait()
        if status != 0:
            sys.exit(status)


def execute_start_script(start_cmd, tsuru_envs):
    exec_with_envs([start_cmd], tsuru_envs, with_shell=True)


def run_hooks(app_data, tsuru_envs):
    commands = (app_data.get('hooks') or {}).get('build') or []
    exec_with_envs(commands, tsuru_envs, with_shell=True)


def load_app_yaml(working_dir="/home/application/current"):
    files_name = ["tsuru.yaml", "tsuru.yml", "app.yaml", "app.yml"]
    for file_name in files_name:
        try:
            with io.open(os.path.join(working_dir, file_name)) as f:
                return yaml.load(f.read())
        except IOError:
            pass
    return None


def save_apprc_file(environs):
    with io.open("/home/application/apprc", "w") as file:
        file.write(u"# generated by tsuru at {}\n".format(datetime.now()))
        for env in environs:
            file.write(u'export {}="{}"\n'.format(env["name"], env["value"]))



from unittest import TestCase
import mock
import os

from tsuru_unit_agent.tasks import execute_start_script, save_apprc_file, run_hooks, load_app_yaml


class TestTasks(TestCase):

    @mock.patch("os.environ", {'env': 'var'})
    @mock.patch("subprocess.Popen")
    def test_execute(self, popen_mock):
        wait_mock = popen_mock.return_value.wait
        wait_mock.return_value = 0
        environs = [
            {"name": "DATABASE_HOST", "value": "localhost", "public": True},
            {"name": "DATABASE_USER", "value": "root", "public": True},
        ]
        execute_start_script("my_command", environs)
        popen_mock.assert_called_with("my_command", shell=True, cwd="/", env={
            "DATABASE_HOST": "localhost",
            "DATABASE_USER": "root",
            "env": "var",
        })
        wait_mock.assert_called_once()

    @mock.patch("os.environ", {'env': 'var'})
    @mock.patch("sys.exit")
    @mock.patch("subprocess.Popen")
    def test_execute_failing(self, popen_mock, exit_mock):
        wait_mock = popen_mock.return_value.wait
        wait_mock.return_value = 10
        environs = [
            {"name": "DATABASE_HOST", "value": "localhost", "public": True},
            {"name": "DATABASE_USER", "value": "root", "public": True},
        ]
        execute_start_script("my_command", environs)
        popen_mock.assert_called_with("my_command", shell=True, cwd="/", env={
            "DATABASE_HOST": "localhost",
            "DATABASE_USER": "root",
            "env": "var"
        })
        wait_mock.assert_called_once()
        exit_mock.assert_called_once_with(10)

    def test_save_apprc_file(self):
        environs = [
            {"name": "DATABASE_HOST", "value": "localhost", "public": True},
            {"name": "DATABASE_USER", "value": "root", "public": True},
        ]

        with mock.patch("io.open", mock.mock_open()) as m:
            save_apprc_file(environs)
            m.assert_called_once_with("/home/application/apprc", "w")
            write_mock = m().write
            self.assertRegexpMatches(write_mock.mock_calls[0][1][0], '# generated by tsuru at .*\n')
            write_mock.assert_any_call('export DATABASE_HOST="localhost"\n')
            write_mock.assert_any_call('export DATABASE_USER="root"\n')
            self.assertEqual(len(write_mock.mock_calls), 3)


class RunHooksTest(TestCase):
    @mock.patch("os.environ", {'env': 'var'})
    @mock.patch("subprocess.Popen")
    def test_execute_commands(self, popen_call):
        wait_mock = popen_call.return_value.wait
        wait_mock.return_value = 0
        data = {"hooks": {"build": ["ble"]}}
        envs = [
            {"name": "my_key", "value": "my_value"},
        ]
        run_hooks(data, envs)
        popen_call.assert_called_with("ble", shell=True,
                                      cwd="/", env={'my_key': 'my_value', 'env': 'var'})
        wait_mock.assert_called_once()

    @mock.patch("os.environ", {})
    @mock.patch("os.path.exists")
    @mock.patch("subprocess.Popen")
    def test_execute_commands_default_cwd_exists(self, popen_call, exists_mock):
        wait_mock = popen_call.return_value.wait
        wait_mock.return_value = 0
        exists_mock.return_value = True
        data = {"hooks": {"build": ["ble"]}}
        envs = [
            {"name": "my_key", "value": "my_value"},
        ]
        run_hooks(data, envs)
        popen_call.assert_called_with("ble", shell=True,
                                      cwd="/home/application/current", env={'my_key': 'my_value'})
        wait_mock.assert_called_once()
        exists_mock.assert_called_once_with("/home/application/current")

    @mock.patch("os.environ", {})
    @mock.patch("sys.exit")
    @mock.patch("subprocess.Popen")
    def test_execute_failing_commands(self, popen_call, exit_mock):
        wait_mock = popen_call.return_value.wait
        wait_mock.return_value = 5
        data = {"hooks": {"build": ["ble"]}}
        envs = [
            {"name": "my_key", "value": "my_value"},
        ]
        run_hooks(data, envs)
        popen_call.assert_called_with("ble", shell=True,
                                      cwd="/", env={'my_key': 'my_value'})
        wait_mock.assert_called_once()
        exit_mock.assert_called_once_with(5)

    @mock.patch("subprocess.Popen")
    def test_execute_commands_hooks_empty(self, subprocess_call):
        data = {}
        run_hooks(data, [])
        subprocess_call.assert_not_called()
        data = {"hooks": None}
        run_hooks(data, [])
        subprocess_call.assert_not_called()
        data = {"hooks": {"build": None}}
        run_hooks(data, [])
        subprocess_call.assert_not_called()
        data = {"hooks": {"build": []}}
        run_hooks(data, [])
        subprocess_call.assert_not_called()


class LoadAppYamlTest(TestCase):
    def setUp(self):
        self.working_dir = os.path.dirname(__file__)
        self.data = '''
hooks:
  build:
    - {0}_1
    - {0}_2'''

    def test_load_app_yaml(self):
        filenames = ["tsuru.yaml", "tsuru.yml", "app.yaml", "app.yml"]
        for name in filenames:
            with open(os.path.join(self.working_dir, name), "w") as f:
                f.write(self.data.format(name))

        for name in filenames:
            data = load_app_yaml(self.working_dir)
            self.assertEqual(data, {"hooks": {"build": ["{}_1".format(name), "{}_2".format(name)]}})
            os.remove(os.path.join(self.working_dir, name))

    def test_load_without_app_files(self):
        data = load_app_yaml(self.working_dir)
        self.assertEqual(data, None)



import unittest
import mock

from tsuru_unit_agent.main import parse_args, main


class TestMain(unittest.TestCase):
    def test_parse_args(self):
        args = parse_args(['a', 'b', 'c', 'd', 'run'])
        self.assertEqual(args.action, 'run')
        self.assertEqual(args.url, 'a')
        self.assertEqual(args.token, 'b')
        self.assertEqual(args.app_name, 'c')
        self.assertEqual(args.start_cmd, 'd')

    def test_parse_args_default_action(self):
        args = parse_args(['a', 'b', 'c', 'd'])
        self.assertEqual(args.action, 'run')
        self.assertEqual(args.url, 'a')
        self.assertEqual(args.token, 'b')
        self.assertEqual(args.app_name, 'c')
        self.assertEqual(args.start_cmd, 'd')

    def test_parse_args_deploy(self):
        args = parse_args(['a', 'b', 'c', 'd', 'deploy'])
        self.assertEqual(args.action, 'deploy')
        self.assertEqual(args.url, 'a')
        self.assertEqual(args.token, 'b')
        self.assertEqual(args.app_name, 'c')
        self.assertEqual(args.start_cmd, 'd')

    def test_parse_args_invalid(self):
        self.assertRaises(SystemExit, parse_args, [])
        self.assertRaises(SystemExit, parse_args, ['a', 'b', 'c', 'd', 'e'])

    @mock.patch('sys.argv', ['', 'http://localhost', 'token', 'app1', 'mycmd', 'deploy'])
    @mock.patch('tsuru_unit_agent.main.tasks')
    @mock.patch('tsuru_unit_agent.main.Client')
    def test_main_deploy_action(self, client_mock, tasks_mock):
        register_mock = client_mock.return_value.register_unit
        register_mock.return_value = [{'name': 'env1', 'value': 'val1'}]
        save_apprc_mock = tasks_mock.save_apprc_file
        exec_script_mock = tasks_mock.execute_start_script
        load_yaml_mock = tasks_mock.load_app_yaml
        load_yaml_mock.return_value = {'hooks': {'build': ['cmd_1', 'cmd_2']}}
        post_app_yaml_mock = client_mock.return_value.post_app_yaml
        run_hooks_mock = tasks_mock.run_hooks
        main()
        call_count = len(client_mock.mock_calls) + len(tasks_mock.mock_calls)
        self.assertEqual(call_count, 7)
        client_mock.assert_called_once_with('http://localhost', 'token')
        register_mock.assert_called_once_with('app1')
        save_apprc_mock.assert_called_once_with(register_mock.return_value)
        exec_script_mock.assert_called_once_with('mycmd', register_mock.return_value)
        load_yaml_mock.assert_called_once()
        post_app_yaml_mock.assert_called_once_with('app1', load_yaml_mock.return_value)
        run_hooks_mock.assert_called_once_with(load_yaml_mock.return_value, register_mock.return_value)

    @mock.patch('sys.argv', ['', 'http://localhost', 'token', 'app1', 'mycmd', 'run'])
    @mock.patch('tsuru_unit_agent.main.tasks')
    @mock.patch('tsuru_unit_agent.main.Client')
    def test_main_run_action(self, client_mock, tasks_mock):
        register_mock = client_mock.return_value.register_unit
        register_mock.return_value = [{'name': 'env1', 'value': 'val1'}]
        save_apprc_mock = tasks_mock.save_apprc_file
        exec_script_mock = tasks_mock.execute_start_script
        main()
        call_count = len(client_mock.mock_calls) + len(tasks_mock.mock_calls)
        self.assertEqual(call_count, 4)
        client_mock.assert_called_once_with('http://localhost', 'token')
        register_mock.assert_called_once_with('app1')
        save_apprc_mock.assert_called_once_with(register_mock.return_value)
        exec_script_mock.assert_called_once_with('mycmd', register_mock.return_value)



import unittest
import mock
from socket import gethostname

from tsuru_unit_agent.client import Client


class TestClient(unittest.TestCase):
    def test_client(self):
        client = Client("http://localhost", "token")
        self.assertEqual(client.url, "http://localhost")
        self.assertEqual(client.token, "token")

    @mock.patch("requests.post")
    def test_register_unit(self, post_mock):
        response = post_mock.return_value
        response.status_code = 200
        response.json = mock.Mock(side_effect=lambda: {"a": "b"})
        client = Client("http://localhost", "token")
        envs = client.register_unit(app="myapp")
        self.assertDictEqual(envs, {"a": "b"})
        post_mock.assert_called_with(
            "{}/apps/myapp/units/register".format(client.url),
            data={"hostname": gethostname()},
            headers={"Authorization": "bearer token"})

    @mock.patch("requests.get")
    @mock.patch("requests.post")
    def test_register_unit_failing_register(self, post_mock, get_mock):
        response = post_mock.return_value
        response.status_code = 404
        response.json.return_value = None
        get_response = get_mock.return_value
        get_response.status_code = 200
        get_response.json.return_value = {"a": "b"}
        client = Client("http://localhost", "token")
        envs = client.register_unit(app="myapp")
        self.assertDictEqual(envs, {"a": "b"})
        post_mock.assert_called_once_with(
            "{}/apps/myapp/units/register".format(client.url),
            data={"hostname": gethostname()},
            headers={"Authorization": "bearer token"})
        get_mock.assert_called_once_with(
            "{}/apps/myapp/env".format(client.url),
            headers={"Authorization": "bearer token"})

    @mock.patch("requests.post")
    def test_post_app_yaml(self, post_mock):
        response = post_mock.return_value
        response.status_code = 200
        client = Client("http://localhost", "token")
        client.post_app_yaml(app="myapp", data=[{"x": "y"}])
        post_mock.assert_called_with(
            "{}/apps/myapp/customdata".format(client.url),
            data='[{"x": "y"}]',
            headers={
                "Authorization": "bearer token",
                "Content-Type": "application/json",
            })



import sys
import argparse

from tsuru_unit_agent import tasks
from tsuru_unit_agent.client import Client


def run_action(args):
    client = Client(args.url, args.token)
    envs = client.register_unit(args.app_name)
    tasks.save_apprc_file(envs)
    tasks.execute_start_script(args.start_cmd, envs)
    return client, envs


def deploy_action(args):
    client, envs = run_action(args)
    yaml_data = tasks.load_app_yaml()
    client.post_app_yaml(args.app_name, yaml_data)
    tasks.run_hooks(yaml_data, envs)


actions = {
    'run': run_action,
    'deploy': deploy_action
}


def parse_args(args=None):
    if args is None:
        args = sys.argv[1:]
    parser = argparse.ArgumentParser(description='Runs tsuru-unit-agent.')
    parser.add_argument('url', help='URL for tsuru API server')
    parser.add_argument('token', help='Authentication token for tsuru API server')
    parser.add_argument('app_name', help='The app name')
    parser.add_argument('start_cmd', help='Command to run after notifying tsuru API server')
    parser.add_argument('action', default='run', nargs='?', choices=actions.keys(), help='Action being executed')
    return parser.parse_args(args)


def main():
    args = parse_args()
    actions[args.action](args)


if __name__ == '__main__':
    main()



from socket import gethostname
import json

import requests


class Client(object):
    def __init__(self, url, token):
        self.url = url
        self.token = token

    def register_unit(self, app):
        params = {
            'headers': {"Authorization": "bearer {}".format(self.token)},
        }
        response = requests.post(
            "{}/apps/{}/units/register".format(self.url, app),
            data={"hostname": gethostname()},
            **params)
        if response.status_code != 200:
            response = requests.get(
                "{}/apps/{}/env".format(self.url, app),
                **params)
        return response.json()

    def post_app_yaml(self, app, data):
        response = requests.post(
            "{}/apps/{}/customdata".format(self.url, app),
            data=json.dumps(data),
            headers={
                "Authorization": "bearer {}".format(self.token),
                "Content-Type": "application/json",
            })
        if not 200 <= response.status_code < 400:
            raise Exception("invalid response {} - {}".format(response.status_code, response.text))
